(function ($) {
  'use strict';

  // Metrics Url.
  if (!drupalSettings.aws_cloud_monitor_metrics_url) {
    return;
  }

  // Format Data.
  const formatData = function (data, item) {
    if (data[item].timestamps) {
      const dataFormated = [];
      data[item].timestamps.forEach((element, index) => {
        dataFormated.push([
          d3.timeParse('%Y-%m-%dT%H:%M:%S+00:00')(data[item].timestamps[index]),
          data[item].values[index],
        ]);
      });
      return dataFormated;
    }
  };

  // Merge Data for 2 streams.
  const mergeData = function (data, data2, names, selectors) {
    return [
      {
        id: names[0],
        values: data,
        selector: selectors[0],
      },
      {
        id: names[1],
        values: data2,
        selector: selectors[1],
      },
    ];
  };

  const colors = [
    ['#edf8e9', '#bae4b3', '#74c476', '#31a354', '#006d2c'],
    ['#e9f8f4', '#b3e4dc', '#74c1c4', '#317fa3', '#003f6d'],
  ];

  // Draw Chart Function.
  const drawChart = function (datas, first) {
    datas.forEach(function (data, idx) {
      // Delete current chart.
      d3.select(data.selector).select('svg').remove();

      // Draw charts.
      if (first) {
        // Delete the axis.
        d3.select('#x-axis').remove();

        // Add <div> for the axis.
        $(data.selector).before('<div id="x-axis"></div>');

        // Draw charts.
        const axis = new HorizonChart();
        axis.setHeight(10);
        axis.setDataSet(data.values);
        axis.setSelector('#x-axis');
        axis.draw();

        const chartg = d3.selectAll('#x-axis g:first-child');
        chartg.style('display', 'none');
        const axisg = d3.selectAll('#x-axis .axis');
        axisg.attr("font-size", "8");
      }

      // Draw charts.
      const horizonChart = new HorizonChart();
      horizonChart.setTitle(data.id);
      horizonChart.setTitleFontSize(10);
      horizonChart.setHeight(20);
      horizonChart.setDataSet(data.values);
      horizonChart.setSelector(data.selector);
      horizonChart.setColors(colors[idx]);
      horizonChart.draw();

      $(data.selector).css('margin-bottom', '0');

      const title = d3.selectAll(data.selector + ' svg text');
      title.attr('y', '70%');

      const g = d3.selectAll(data.selector + ' .axis');
      g.style('display', 'none');
      if (!first) {
        $(data.selector).css('border-top', '1px solid #ddd');
      }
      const axis = d3.selectAll(data.selector + ' .axis text');
      axis.attr('y', '22%');
    });
  };

  // Init Chart.
  const initCharts = function () {
    d3.json(drupalSettings.aws_cloud_monitor_metrics_url).then((rawData) => {
      // Get Data.
      const cpuData = [{ id: 'CPU', values: formatData(rawData, 'cpu'), selector: '#edit-cpu-chart'}];
      const networkData = mergeData(
        formatData(rawData, 'network_in'),
        formatData(rawData, 'network_out'),
        ['NetworkIn', 'NetworkOut'],
        ['#edit-network-in-chart', '#edit-network-out-chart']
      );
      const diskData = mergeData(
        formatData(rawData, 'disk_read'),
        formatData(rawData, 'disk_write'),
        ['DiskRead', 'DiskWrite'],
        ['#edit-disk-read-chart', '#edit-disk-write-chart']
      );
      const diskOperationData = mergeData(
        formatData(rawData, 'disk_read_operation'),
        formatData(rawData, 'disk_write_operation'),
        ['DiskOperationRead', 'DiskOperationWrite'],
        ['#edit-disk-read-operation-chart', '#edit-disk-write-operation-chart']
      );

      // Draw Chart.
      drawChart(cpuData, true);
      drawChart(networkData);
      drawChart(diskData);
      drawChart(diskOperationData);
    });
  }

  // Init Charts.
  initCharts();

  // Refresh charts.
  const interval = drupalSettings.aws_cloud_monitor_refresh_interval || 10;
  setInterval(function () {
    initCharts();
  }, interval * 1000);

})(jQuery);
