<?php

namespace Drupal\aws_cloud\Traits;

/**
 * The trait for AWS Cloud form.
 */
trait AwsCloudFormTrait {

  /**
   * Max length of the description of AWS Cloud security group (rule).
   *
   * @var int
   */
  public static $awsCloudSecurityGroupDescriptionMaxLength = 255;

  /**
   * Special characters in the description of AWS Cloud security group (rule).
   *
   * @var string
   */
  public static $awsCloudSecurityGroupDescriptionSpecialCharacters = '. _-:/()#,@[]+=&;{}!$*';

  /**
   * Get the pattern of a security group (rule) description.
   *
   * @return string
   *   A regexp pattern excluding ^ and $.
   */
  protected function getPatternOfSecurityGroupDescription(): string {
    return '[A-Za-z0-9' . preg_quote(self::$awsCloudSecurityGroupDescriptionSpecialCharacters, NULL) . ']*';
  }

  /**
   * Get VPC options.
   *
   * @param string $cloud_context
   *   The cloud context.
   *
   * @return array
   *   The VPC options.
   */
  protected function getVpcOptions(string $cloud_context): array {
    $options = [];
    $vpcs = $this->entityTypeManager
      ->getStorage('aws_cloud_vpc')
      ->loadByProperties([
        'cloud_context' => $cloud_context,
      ]);

    foreach ($vpcs ?: [] as $vpc) {
      $name = NULL;
      foreach ($vpc->getTags() ?: [] as $tag) {
        if ($tag['item_key'] === 'Name') {
          $name = $tag['item_value'];
          break;
        }
      }

      $options[$vpc->getVpcId()] = ($name ?: $vpc->getVpcId()) . " ({$vpc->getVpcId()} | {$vpc->getCidrBlock()})";
    }

    return $options;
  }

}
