Feature: Launch EC2 instance as "Authenticated User"
  In order to launch EC2 instance
  As a user
  I need to create necessary resource

  @api
  Scenario: Log into the site
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds"
    Then I should see the heading "Cloud service providers"
    And I should see the link "All" in the "nav_bar"
    And I should see the link "AWS" in the "nav_bar"

  @api
  Scenario: Create an SSH key pair
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/key_pair/add"
    And I should see the heading "Add AWS Cloud key pair"
    And I enter "{{ ssh_key_name }}" for "Key pair name"
    And I press "Save"
    Then the url should match "/clouds/aws_cloud/{{ cloud_context }}/key_pair"
    And I should see "has been created"
    And I should see the link "{{ ssh_key_name }}"

  @javascript @api
  Scenario: Import an AMI image
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/images/import"
    And I enter "{{ image_id }}" for "Image IDs"
    And I press "Import"
    And I wait for AJAX to finish
    Then I should be on "/clouds/aws_cloud/{{ cloud_context }}/image"
    And I should see "Imported 1 images"
    And I should see the link "{{ image_name }}" in the table

  @api
  Scenario: Add a VPC
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/vpc/add"
    And I enter "{{ vpc_name }}" for "Name"
    And I enter "{{ ipv4_cidr }}" for "IPv4 CIDR block"
    And I press "Save"
    Then I should be on "/clouds/aws_cloud/{{ cloud_context }}/vpc"
    And I should see "has been created"
    And I should see the link "{{ vpc_name }}" in the table

  @javascript @api
  Scenario: Add a subnet
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/subnet/add"
    And I enter "{{ subnet_name }}" for "Name"
    And I select "{{ vpc_name }}" from "VPC CIDR (ID)"
    And I select "{{ avail_zone }}" from "Availability Zone"
    And I enter "{{ ipv4_cidr }}" for "IPv4 CIDR block"
    And I press "Save"
    And I wait for AJAX to finish
    Then I should be on "/clouds/aws_cloud/{{ cloud_context }}/subnet"
    And I should see "has been created"
    And I should see the link "{{ subnet_name }}" in the table

  @javascript @api
  Scenario: Add a security group
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/security_group/add"
    And I enter "{{ security_group_name }}" for "Security group name"
    And I enter "{{ description }}" for "Description"
    And I select "{{ vpc_name }}" from "VPC CIDR (ID)"
    And I press "Save"
    Then the url should match "/clouds/aws_cloud/{{ cloud_context }}/security_group/"
    And I should see "has been created"
    And I click "Inbound and Outbound Rules"
    And the url should match "/edit"
    And I select "{{ ip_protocol }}" from "IP Protocol"
    And I enter "{{ ipv4_cidr }}" for "CIDR IP"
    And I press "Save"
    And I wait for AJAX to finish
    And the url should match "/clouds/aws_cloud/{{ cloud_context }}/security_group/"
    And I should see "has been updated"
    And I should see the link "{{ security_group_name }}"

  @javascript @api
  Scenario: Create a launch template
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/design/server_template/{{ cloud_context }}/aws_cloud/add"
    And I should see the heading "Add AWS Cloud"
    And I enter "{{ instance_name }}" for "Name"
    And I select "{{ instance_type }}" from "Instance type"
    And I select "{{ image_name }}" from "Image ID"
    And I select "{{ avail_zone }}" from "Availability Zone"
    And I select "{{ vpc_name }}" from "VPC"
    And I wait for AJAX to finish
    And I select "{{ subnet_name }}" from "Subnet"
    And I select "{{ security_group_name }}" from "Security groups"
    And I select "{{ ssh_key_name }}" from "SSH key"
    And I select "{{ status }}" from "Status"
    And I press "Save"
    And I wait for AJAX to finish
    Then the url should match "/clouds/design/server_template/{{ cloud_context }}/"
    And I should see "has been created"
    And I should see the heading "{{ instance_name }}"
    And I visit "/clouds/design/server_template/{{ cloud_context }}"
    And I should see the link "{{ instance_name }}" in the table

  @javascript @api
  Scenario: Launch an instance
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/design/server_template/{{ cloud_context }}"
    And I click "{{ instance_name }}"
    And I should see the heading "{{ instance_name }}"
    And I click "Approve"
    And the url should match "/approve"
    And I press "Approve"
    And the url should match "/clouds/design/server_template/{{ cloud_context }}/"
    And I should see "has been approved"
    When I click "Launch"
    And the url should match "/launch"
    And I check the box "Automatically terminate instance"
    And I press "Launch"
    Then I should be on "/clouds/aws_cloud/{{ cloud_context }}/instance"
    And I should see "pending" in the "{{ instance_name }}" row

  @api
  Scenario: View the instance
    Given I am logged in as a user with the "{{ role_name }}" role
    When I visit "/clouds/aws_cloud/{{ cloud_context }}/instance"
    And I click "{{ instance_name }}"
    Then the url should match "/clouds/aws_cloud/{{ cloud_context }}/instance/"
    And I should see the link "Info"
    And I should see the link "Monitor"
    And I should see the link "Console Output"
