Introduction
============

Behat is an open source Behat-Driven Development framework for PHP. It is a tool
to support you in delivering software that matters through continuous
communication, deliberate discovery and test-automation.

Features and Scenarios in Behat
-------------------------------
Every `*.feature` file conventionally consists of a single feature. A feature
usually contains a list of scenarios.
Every scenario consists of a list of steps, which must start with one of
the keywords `Given`, `When`, `Then`, `And`, or `But`.

Behat structure in our project
------------------------------
We have 2 Behat directories:
1. `./modules/cloud_service_providers/aws_cloud/tests/src/Behat`
   has Behat features specifically for AWS.
2. `./tests/src/Behat` has general Behat features.

We need to run `behat *.feature` under `./Behat`

Something we should know about Behat
------------------------------------
1. When running Behat, we are actually accessing, creating and deleting
   resources. As a result, accessing the site on the browser and the container
   you are working on may be slow. We can speed things up in the future by
   - optimizing the test code
   - running multiple tests at the same time
   - getting better hardware
   - optimizing the application (Cloud Orchestrator) speed
   - running just the highest priority on build time
3. It is important to run `drush cr` when running Behat to clear cache.
   We should run it at anywhere under e.g. `/var/www/html/web`
4. When the `shm-size` (shared memory size) of the container is too low,
   chrome (the web driver) is likely to crash randomly. When this happens,
   the driver only reports a timeout in the connection. For the existing Behat
   files, it is safe to use a container with `shm-size=256m`

-------------------------------

Set up environment to run Behat
===============================
For any service, we can run `<service-name> -V` or `<service-name> --version`
to check if the service is successfully setup

1. If you already have a container with `shm-size >= 256m`
   (run `df -h /dev/shm` to display your shared memory size), 
   please go to step 2. 
   Else, please create a container with the following command.

   ```
   cd; vi start.sh
   ```
   ```
   #!/bin/bash

   IMAGE_TAG=docomoinnovations/cloud_orchestrator
   CONTAINER_NAME=cloud-orchestrator-qa

   docker start cloud-orchestrator-memcache
   docker start mariadb

   CO_DIR="$HOME/cloud_orchestrator"
   DOCKER_DIR="/var/www/cloud_orchestrator"

   sudo docker run --name $CONTAINER_NAME --link mariadb:mysql \
                   --link cloud-orchestrator-memcache:memcache \
                   --shm-size=256m  \
                   -p 8080:80 -d -v $CO_DIR:$DOCKER_DIR $IMAGE_TAG
   ```
   ```
   chmod u+x start.sh; ./start.sh
   docker exec -it <your-container-id> bash
   ```

2. Set up services/tools required

   1. **vim**
      ```
      apt-get update
      apt-get install vim
      vim --verion
      ```

   2. **wget**
      ```
      apt-get install wget
      wget -V
      ```

   3. **ssh-client**
      ```
      apt-get install ssh-client
      ```

   4. **net-tools**
      ```
      apt-get install net-tools
      netstat -tulpn
      ```

   5. **drush**

      Check if drush works well by running:
      ```
      drush cr; drush updb -y
      ```

      If encounter this error
      `You have requested a non-existent service "cached.backend.memcached"`,
      comment out the last line of `settings.php`:
      ```
      find -name settings.php
      vi <path-to>/settings.php
      ...
      // $settings['cache']['default'] = 'cache.backend.memcache';
      ```
      And then setup memcached by running the following. We may need to exit
      the docker, or uninstall and reinstall memcached if service memcached
      status did not return desired output)
      ```
      apt-get install memcached
      /etc/init.d/memcached restart
      service memcached status
      ```

   6. **behat**

      Get to the level where it has: the directory `vendor`, the files
      `composer.json` and `composer.lock`
      ```
      composer require --dev drupal/drupal-extension
      ```
      Add `$PATH` in `~/.bashrc`
      ```
      export PATH=$PATH:/var/www/cloud_orchestrator/vendor/bin
      ```
      Exit the container and re-enter

   7. **chromium/chrome-driver**
      
      We need to go to the same directory where we setup behat
      ```
      composer require --dev dmore/behat-chrome-extension
      apt-get install chromium-driver
      ```
      Run chromium in headless mode
      ```
      chromium --disable-gpu --headless --no-sandbox \
               --remote-debugging-address=0.0.0.0 \
               --remote-debugging-port=9222
      ```
      Use the following command to check if chromium is running
      ```
      netstat -tulpn
      ```

3. Modify `DevToolsConnection.php` to modify how `chrome-mink-driver` deals with 
   how DevToolsConnections handles timeout.
   
   We need to modify the function 
   `protected function waitFor(callable $is_ready)` in 
   `<path-to>/dmore/chrome-mink-driver/src/DevToolsConnection.php` so that 
   it will re-try after failing to connect.
   
   Please refer to the patch at [./tests/src/Behat/DevToolsConnection.patch](
     https://git.drupalcode.org/project/cloud/-/blob/4.x/tests/src/Behat/DevToolsConnection.patch
   ) and replace `DevToolsConnection.php` with it.


4. Manually set `BEHAT_PARAMS`
   ```
   export BEHAT_PARAMS='{
     "extensions": {
       "Drupal\\MinkExtension": {
         "base_url": "<your-url>"
       },
       "Drupal\\DrupalExtension": {
         "drupal": {
           "drupal_root": "<path-to-modules/contrib/cloud>"
         }
       }
     }
   }'
   ```

----------------

How to run Behat
================
1. Find where the `behat.yml` locates, and go there.
2. Run `behat <path-to>/*.feature` to run a spcific `*.feature` file. 
   For example,
   ```
   behat features/test.feature
   ```
3. Run 'behat' to run all your `*.feature` files in alphabetically order under 
   your current path.
