Feature: Login, Check the Clean State, and Logout
  In order to list resources on the clean state
  As a user
  I need to have both Drupal and AWS IAM permissions

  Scenario: Log in as "Anonymous user"
    Given I am not logged in
    When I visit "/clouds"
    Then I should be on "/user/login"
    And I should see the heading "Log in"

  @api
  Scenario Outline: Log in as "Authenticated user" and "Administrator"
    Given I am logged in as a user with the <role> role
    And I should not see the link "All" in the "nav_bar"
    When I go to "/clouds"
    Then I should get a <code> HTTP response
    And I should see the heading <heading>

    Examples:
      |  role                | code | heading                      |
      | "Authenticated user" | 403  | "Access denied"              |
      | "Administrator"      | 200  | "Add cloud service provider" |

  @api
  Scenario: Log in as "Administrator" and see if no cloud service provider
    Given I am logged in as a user with the "Administrator" role
    And I should not see the link "All" in the "nav_bar"
    When I visit "/clouds"
    Then I should be on "/admin/structure/cloud_config/add"
    And I should see "There is no cloud service provider."
    And I should see the link "AWS Cloud"

  @api
  Scenario Outline: Log out as "Authenticated user" and "Administrator"
    Given I am logged in as a user with the <role> role
    And I should not see the link "All" in the "nav_bar"
    And I visit "/"
    And I see the heading "Cloud service provider location map"
    When I click "Log out"
    Then I should be on "/user/login"
    And I should see the link "Log in"

    Examples:
      | role                 |
      | "Authenticated user" |
      | "Administrator"      |

  @api
  Scenario Outline: View the list of cloud service providers
    Given I am logged in as a user with the <role> role
    And I should not see the link "All" in the "nav_bar"
    When I go to "/clouds"
    Then I should be on "/admin/structure/cloud_config/add"
    And I should get a <code> HTTP response
    And I should see the heading <heading>

  Examples:
    | role                 | code | heading                      |
    | "Authenticated user" | 403  | "Access denied"              |
    | "Administrator"      | 200  | "Add cloud service provider" |

  @api
  Scenario: "Administrator" can view the list of cloud service providers
    Given I am logged in as a user with the "Administrator" role
    And I should not see the link "All" in the "nav_bar"
    When I visit "/clouds"
    Then I should be on "/admin/structure/cloud_config/add"
    And I should see "There is no cloud service provider."
    And I should see the link "AWS Cloud"
    And I should see the link "K8s"
    And I should see the link "VMware"
